@@ -21,16 +21,19 @@
 #include "qgis_core.h"
 
 #include "qgsrasterlayer.h"
-#include "qgsvectorlayer.h"
 #include "qgsmessagelog.h"
 #include "qgsspatialindex.h"
 #include "qgsprocessing.h"
+#include "qgsfeaturesink.h"
+#include "qgsfeaturesource.h"
 
+class QgsMeshLayer;
 class QgsProject;
 class QgsProcessingContext;
 class QgsMapLayerStore;
 class QgsProcessingFeedback;
 class QgsProcessingFeatureSource;
+class QgsProcessingAlgorithm;
 
 #include <QString>
 #include <QVariant>
@@ -43,7 +46,6 @@ class QgsProcessingFeatureSource;
  */
 class CORE_EXPORT QgsProcessingUtils
 {
-
   public:
 
     /**
@@ -53,6 +55,7 @@ class CORE_EXPORT QgsProcessingUtils
      * If the \a sort argument is true then the layers will be sorted by their QgsMapLayer::name()
      * value.
      * \see compatibleVectorLayers()
+     * \see compatibleMeshLayers()
      * \see compatibleLayers()
      */
     static QList< QgsRasterLayer * > compatibleRasterLayers( QgsProject *project, bool sort = true );
@@ -69,12 +72,28 @@ class CORE_EXPORT QgsProcessingUtils
      * If the \a sort argument is true then the layers will be sorted by their QgsMapLayer::name()
      * value.
      * \see compatibleRasterLayers()
+     * \see compatibleMeshLayers()
      * \see compatibleLayers()
      */
     static QList< QgsVectorLayer * > compatibleVectorLayers( QgsProject *project,
         const QList< int > &sourceTypes = QList< int >(),
         bool sort = true );
 
+    /**
+     * Returns a list of mesh layers from a \a project which are compatible with the processing
+     * framework.
+     *
+     * If the \a sort argument is true then the layers will be sorted by their QgsMapLayer::name()
+     * value.
+     *
+     * \see compatibleRasterLayers()
+     * \see compatibleVectorLayers()
+     * \see compatibleLayers()
+     *
+     * \since QGIS 3.6
+     */
+    static QList<QgsMeshLayer *> compatibleMeshLayers( QgsProject *project, bool sort = true );
+
     /**
      * Returns a list of map layers from a \a project which are compatible with the processing
      * framework.
@@ -95,6 +114,7 @@ class CORE_EXPORT QgsProcessingUtils
       UnknownType, //!< Unknown layer type
       Vector, //!< Vector layer type
       Raster, //!< Raster layer type
+      Mesh, //!< Mesh layer type  \since QGIS 3.6
     };
 
     /**
@@ -130,8 +150,18 @@ class CORE_EXPORT QgsProcessingUtils
      */
     static QString normalizeLayerSource( const QString &source );
 
+    /**
+     * Converts a variant to a Python literal.
+     *
+     * \see stringToPythonLiteral()
+     * \since QGSIS 3.6
+     */
+    static QString variantToPythonLiteral( const QVariant &value );
+
     /**
      * Converts a string to a Python string literal. E.g. by replacing ' with \'.
+     *
+     * \see variantToPythonLiteral()
      */
     static QString stringToPythonLiteral( const QString &string );
 
@@ -153,13 +183,13 @@ class CORE_EXPORT QgsProcessingUtils
      * The caller takes responsibility for deleting the returned sink.
      */
 #ifndef SIP_RUN
-    static QgsFeatureSink *createFeatureSink(
-      QString &destination,
-      QgsProcessingContext &context,
-      const QgsFields &fields,
-      QgsWkbTypes::Type geometryType,
-      const QgsCoordinateReferenceSystem &crs,
-      const QVariantMap &createOptions = QVariantMap() ) SIP_FACTORY;
+    static QgsFeatureSink *createFeatureSink( QString &destination,
+        QgsProcessingContext &context,
+        const QgsFields &fields,
+        QgsWkbTypes::Type geometryType,
+        const QgsCoordinateReferenceSystem &crs,
+        const QVariantMap &createOptions = QVariantMap(),
+        QgsFeatureSink::SinkFlags sinkFlags = nullptr ) SIP_FACTORY;
 #endif
 
     /**
@@ -263,8 +293,8 @@ class CORE_EXPORT QgsProcessingUtils
     static QgsFields indicesToFields( const QList<int> &indices, const QgsFields &fields );
 
   private:
-
     static bool canUseLayer( const QgsRasterLayer *layer );
+    static bool canUseLayer( const QgsMeshLayer *layer );
     static bool canUseLayer( const QgsVectorLayer *layer,
                              const QList< int > &sourceTypes = QList< int >() );
 
@@ -330,6 +360,8 @@ class CORE_EXPORT QgsProcessingFeatureSource : public QgsFeatureSource
      */
     QgsFeatureIterator getFeatures( const QgsFeatureRequest &request, Flags flags ) const;
 
+    QgsFeatureSource::FeatureAvailability hasFeatures() const override;
+
     QgsFeatureIterator getFeatures( const QgsFeatureRequest &request = QgsFeatureRequest() ) const override;
     QgsCoordinateReferenceSystem sourceCrs() const override;
     QgsFields fields() const override;
