@@ -23,7 +23,7 @@ __copyright__ = '(C) 2013, Alexander Bruy'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '1edf372fb80f2a35c77b128dc143c0f3c97cd8f3'
+__revision__ = '58734527abdbaab95719a48a4a64ea05abd70b44'
 
 import os
 
@@ -62,7 +62,7 @@ class ScriptEdit(QsciScintilla):
     FOLD_COLOR = "#efefef"
 
     def __init__(self, parent=None):
-        QsciScintilla.__init__(self, parent)
+        super().__init__(parent)
 
         self.lexer = None
         self.api = None
@@ -128,7 +128,7 @@ class ScriptEdit(QsciScintilla):
         self.setTabWidth(4)
         self.setIndentationGuides(True)
 
-        # Autocomletion
+        # Autocompletion
         self.setAutoCompletionThreshold(2)
         self.setAutoCompletionSource(QsciScintilla.AcsAPIs)
 
@@ -145,8 +145,6 @@ class ScriptEdit(QsciScintilla):
         self.defaultFont.setFixedPitch(True)
         self.defaultFont.setPointSize(fontSize)
         self.defaultFont.setStyleHint(QFont.TypeWriter)
-        self.defaultFont.setStretch(QFont.SemiCondensed)
-        self.defaultFont.setLetterSpacing(QFont.PercentageSpacing, 87.0)
         self.defaultFont.setBold(False)
 
         self.boldFont = QFont(self.defaultFont)
